=== modified file 'HACKING'
--- HACKING	2005-10-06 22:15:52 +0000
+++ HACKING	2005-10-17 07:48:02 +0000
@@ -178,7 +178,10 @@
 Errors and exceptions
 =====================
 
-[write me]
+Errors are handled through Python exceptions.  They can represent user
+errors, environmental errors or program bugs.  Sometimes we can't be sure
+at the time it's raised which case applies.  See bzrlib/errors.py for 
+details on the error-handling practices.
 
 
 Jargon

=== modified file 'NEWS'
--- NEWS	2005-10-18 12:07:55 +0000
+++ NEWS	2005-10-18 13:11:57 +0000
@@ -2,6 +2,10 @@
 
   IMPROVEMENTS:
 
+    * New 'bzr commit --strict' option refuses to commit if there are 
+      any unknown files in the tree.  To commit, make sure all files are 
+      either ignored, added, or deleted.  (Michael Ellerman)
+
     * The config directory is now ~/.bazaar, and there is a single file 
       ~/.bazaar/bazaar.conf storing email, editor and other preferences.
       (Robert Collins)
@@ -40,17 +44,21 @@
       related questions. (Robert Collins)
 
     * 'bzr selftest' runs all tests, even if one feels, unless '--one'
-      is given.
+      is given. (Martin Pool)
 
     * There is a new method for TestCaseInTempDir, assertFileEqual, which
       will check that a given content is equal to the content of the named
-      file.
+      file. (Robert Collins)
 
   INTERNALS:
 
-    * New 'manifest' command and concept for making gpg-signatures 
+    * New 'testament' command and concept for making gpg-signatures 
       of revisions that are not tied to a particular internal
-      representation.
+      representation.  (Martin Pool).
+
+    * Per-revision properties ('revprops') as key-value associated 
+      strings on each revision created when the revision is committed.
+      Intended mainly for the use of external tools.  (Martin Pool).
 
     * Config options have moved from bzrlib.osutils to bzrlib.config.
       (Robert Collins)
@@ -76,6 +84,9 @@
       parameter, and if None is given will open the branch at basedir 
       implicitly. (Robert Collins)
 
+    * Cleaner exception structure and error reporting.  Suggested by 
+      Scott James Remnant.  (Martin Pool)
+
     * Branch.remove has been moved to WorkingTree, which has also gained
       lock_read, lock_write and unlock methods for convenience. (Robert
       Collins)
@@ -99,6 +110,10 @@
       options of built in programs should have specific methods on the config.
       (Robert Collins)
 
+  BUG FIXES:
+
+    * Better handling of branches in directories with non-ascii names. 
+      (Joel Rosdahl, Panagiotis Papadakos)
 
 bzr 0.1.1 2005-10-12
 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-18 06:56:24 +0000
+++ bzrlib/branch.py	2005-10-18 13:11:57 +0000
@@ -28,12 +28,12 @@
 from bzrlib.trace import mutter, note
 from bzrlib.osutils import (isdir, quotefn, compact_date, rand_bytes, 
                             rename, splitpath, sha_file, appendpath, 
-                            file_kind)
+                            file_kind, abspath)
 import bzrlib.errors as errors
 from bzrlib.errors import (BzrError, InvalidRevisionNumber, InvalidRevisionId,
                            NoSuchRevision, HistoryMissing, NotBranchError,
                            DivergedBranches, LockError, UnlistableStore,
-                           UnlistableBranch, NoSuchFile)
+                           UnlistableBranch, NoSuchFile, NotVersionedError)
 from bzrlib.textui import show_status
 from bzrlib.revision import Revision, is_ancestor, get_intervening_revisions
 
@@ -136,7 +136,7 @@
             new_t = t.clone('..')
             if new_t.base == t.base:
                 # reached the root, whatever that may be
-                raise NotBranchError('%s is not in a branch' % url)
+                raise NotBranchError(path=url)
             t = new_t
 
     @staticmethod
@@ -496,7 +496,7 @@
         try:
             fmt = self.controlfile('branch-format', 'r').read()
         except NoSuchFile:
-            raise NotBranchError(self.base)
+            raise NotBranchError(path=self.base)
         mutter("got branch format %r", fmt)
         if fmt == BZR_BRANCH_FORMAT_6:
             self._branch_format = 6
@@ -657,16 +657,17 @@
 
         These are files in the working directory that are not versioned or
         control files or ignored.
+        
         >>> from bzrlib.workingtree import WorkingTree
         >>> b = ScratchBranch(files=['foo', 'foo~'])
-        >>> list(b.unknowns())
+        >>> map(str, b.unknowns())
         ['foo']
         >>> b.add('foo')
         >>> list(b.unknowns())
         []
         >>> WorkingTree(b.base, b).remove('foo')
         >>> list(b.unknowns())
-        ['foo']
+        [u'foo']
         """
         return self.working_tree().unknowns()
 
@@ -949,7 +950,7 @@
     def working_tree(self):
         """Return a `Tree` for the working copy."""
         from bzrlib.workingtree import WorkingTree
-        # TODO: In the future, WorkingTree should utilize Transport
+        # TODO: In the future, perhaps WorkingTree should utilize Transport
         # RobertCollins 20051003 - I don't think it should - working trees are
         # much more complex to keep consistent than our careful .bzr subset.
         # instead, we should say that working trees are local only, and optimise
@@ -1082,7 +1083,6 @@
             If true (default) backups are made of files before
             they're renamed.
         """
-        from bzrlib.errors import NotVersionedError, BzrError
         from bzrlib.atomicfile import AtomicFile
         from bzrlib.osutils import backup_file
         
@@ -1095,7 +1095,7 @@
         for fn in filenames:
             file_id = inv.path2id(fn)
             if not file_id:
-                raise NotVersionedError("not a versioned file", fn)
+                raise NotVersionedError(path=fn)
             if not old_inv.has_id(file_id):
                 raise BzrError("file not present in old tree", fn, file_id)
             nids.append((fn, file_id))

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-18 06:56:24 +0000
+++ bzrlib/builtins.py	2005-10-18 13:11:57 +0000
@@ -985,7 +985,9 @@
     """
     # TODO: Run hooks on tree to-be-committed, and after commit.
 
-    # TODO: Strict commit that fails if there are unknown or deleted files.
+    # TODO: Strict commit that fails if there are deleted files.
+    #       (what does "deleted files" mean ??)
+
     # TODO: Give better message for -s, --summary, used by tla people
 
     # XXX: verbose currently does nothing
@@ -997,12 +999,16 @@
                      Option('file', type=str, 
                             argname='msgfile',
                             help='file containing commit message'),
+                     Option('strict',
+                            help="refuse to commit if there are unknown "
+                            "files in the working tree."),
                      ]
     aliases = ['ci', 'checkin']
 
     def run(self, message=None, file=None, verbose=True, selected_list=None,
-            unchanged=False):
-        from bzrlib.errors import PointlessCommit, ConflictsInTree
+            unchanged=False, strict=False):
+        from bzrlib.errors import (PointlessCommit, ConflictsInTree,
+                StrictCommitFailed)
         from bzrlib.msgeditor import edit_commit_message
         from bzrlib.status import show_status
         from cStringIO import StringIO
@@ -1031,9 +1037,8 @@
                 raise BzrCommandError("empty commit message specified")
             
         try:
-            b.commit(message,
-                     specific_files=selected_list,
-                     allow_pointless=unchanged)
+            b.commit(message, specific_files=selected_list,
+                     allow_pointless=unchanged, strict=strict)
         except PointlessCommit:
             # FIXME: This should really happen before the file is read in;
             # perhaps prepare the commit; get the message; then actually commit
@@ -1042,6 +1047,9 @@
         except ConflictsInTree:
             raise BzrCommandError("Conflicts detected in working tree.  "
                 'Use "bzr conflicts" to list, "bzr resolve FILE" to resolve.')
+        except StrictCommitFailed:
+            raise BzrCommandError("Commit refused because there are unknown "
+                                  "files in the working tree.")
 
 
 class cmd_check(Command):

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-10-17 21:57:32 +0000
+++ bzrlib/commit.py	2005-10-18 06:30:47 +0000
@@ -56,6 +56,12 @@
 # merges from, then it should still be reported as newly added
 # relative to the basis revision.
 
+# TODO: Do checks that the tree can be committed *before* running the 
+# editor; this should include checks for a pointless commit and for 
+# unknown or missing files.
+
+# TODO: If commit fails, leave the message in a file somewhere.
+
 
 import os
 import re
@@ -75,7 +81,8 @@
 import bzrlib.config
 from bzrlib.errors import (BzrError, PointlessCommit,
                            HistoryMissing,
-                           ConflictsInTree
+                           ConflictsInTree,
+                           StrictCommitFailed
                            )
 import bzrlib.gpg as gpg
 from bzrlib.revision import Revision
@@ -166,6 +173,7 @@
                specific_files=None,
                rev_id=None,
                allow_pointless=True,
+               strict=False,
                verbose=False,
                revprops=None):
         """Commit working copy as a new revision.
@@ -184,6 +192,9 @@
         allow_pointless -- If true (default), commit even if nothing
             has changed and no merges are recorded.
 
+        strict -- If true, don't allow a commit if the working tree
+            contains unknown files.
+
         revprops -- Properties for new revision
         """
         mutter('preparing to commit')
@@ -195,6 +206,9 @@
         self.allow_pointless = allow_pointless
         self.revprops = revprops
 
+        if strict and branch.unknowns():
+            raise StrictCommitFailed()
+
         if timestamp is None:
             self.timestamp = time.time()
         else:

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-10-17 11:56:54 +0000
+++ bzrlib/errors.py	2005-10-18 06:30:47 +0000
@@ -14,24 +14,54 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
-__copyright__ = "Copyright (C) 2005 Canonical Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-
-# TODO: Change to a standard exception pattern: 
-#
-# - docstring of exceptions is a template for formatting the exception
-#   so the __str__ method can be defined only in the superclass
-# - the arguments to the exception are interpolated into this string
-#
-# when printing the exception we'd then require special handling only
-# for built-in exceptions with no decent __str__ method, such as 
-# ValueError and AssertionError.  See 
-# scott@canonical.com--2005/hct--devel--0.10 util/errors.py
-
-
-######################################################################
-# exceptions 
+"""Exceptions for bzr, and reporting of them.
+
+Exceptions are caught at a high level to report errors to the user, and
+might also be caught inside the program.  Therefore it needs to be
+possible to convert them to a meaningful string, and also for them to be
+interrogated by the program.
+
+Exceptions are defined such that the arguments given to the constructor
+are stored in the object as properties of the same name.  When the
+object is printed as a string, the doc string of the class is used as
+a format string with the property dictionary available to it.
+
+This means that exceptions can used like this:
+
+>>> import sys
+>>> try:
+...   raise NotBranchError(path='/foo/bar')
+... except:
+...   print sys.exc_type
+...   print sys.exc_value
+...   print sys.exc_value.path
+bzrlib.errors.NotBranchError
+Not a branch: /foo/bar
+/foo/bar
+
+Therefore:
+
+ * create a new exception class for any class of error that can be
+   usefully distinguished.
+
+ * the printable form of an exception is generated by the base class
+   __str__ method
+"""
+
+# based on Scott James Remnant's hct error classes
+
+# TODO: is there any value in providing the .args field used by standard
+# python exceptions?   A list of values with no names seems less useful 
+# to me.
+
+# TODO: Perhaps convert the exception to a string at the moment it's 
+# constructed to make sure it will succeed.  But that says nothing about
+# exceptions that are never raised.
+
+# TODO: Convert all the other error classes here to BzrNewError, and eliminate
+# the old one.
+
+
 class BzrError(StandardError):
     def __str__(self):
         # XXX: Should we show the exception class in 
@@ -51,17 +81,43 @@
             return n + `self.args`
 
 
-class BzrCheckError(BzrError):
-    pass
-
-
-class InvalidRevisionNumber(BzrError):
+class BzrNewError(BzrError):
+    """bzr error"""
+    # base classes should override the docstring with their human-
+    # readable explanation
+
+    def __init__(self, **kwds):
+        for key, value in kwds.items():
+            setattr(self, key, value)
+
     def __str__(self):
-        return 'invalid revision number: %r' % self.args[0]
-
-
-class InvalidRevisionId(BzrError):
-    pass
+        try:
+            return self.__doc__ % self.__dict__
+        except (NameError, ValueError, KeyError), e:
+            return 'Unprintable exception %s: %s' \
+                % (self.__class__.__name__, str(e))
+
+
+class BzrCheckError(BzrNewError):
+    """Internal check failed: %(message)s"""
+    def __init__(self, message):
+        self.message = message
+
+
+class InvalidEntryName(BzrNewError):
+    """Invalid entry name: %(name)s"""
+    def __init__(self, name):
+        self.name = name
+
+
+class InvalidRevisionNumber(BzrNewError):
+    """Invalid revision number %(revno)d"""
+    def __init__(self, revno):
+        self.revno = revno
+
+
+class InvalidRevisionId(BzrNewError):
+    """Invalid revision-id"""
 
 
 class BzrCommandError(BzrError):
@@ -69,11 +125,14 @@
     def __str__(self):
         return self.args[0]
 
+class StrictCommitFailed(Exception):
+    """Commit refused because there are unknowns in the tree."""
 
-class NotBranchError(BzrError):
-    """Specified path is not in a branch"""
-    def __str__(self):
-        return 'not a branch: %s' % self.args[0]
+class NotBranchError(BzrNewError):
+    """Not a branch: %(path)s"""
+    def __init__(self, path):
+        BzrNewError.__init__(self)
+        self.path = path
 
 
 class UnsupportedFormatError(BzrError):
@@ -82,33 +141,28 @@
         return 'unsupported branch format: %s' % self.args[0]
 
 
-class NotVersionedError(BzrError):
-    """Specified object is not versioned."""
+class NotVersionedError(BzrNewError):
+    """%(path)s is not versioned"""
+    def __init__(self, path):
+        BzrNewError.__init__(self)
+        self.path = path
 
 
 class BadFileKindError(BzrError):
     """Specified file is of a kind that cannot be added.
 
     (For example a symlink or device file.)"""
-    pass
 
 
 class ForbiddenFileError(BzrError):
     """Cannot operate on a file because it is a control file."""
-    pass
 
 
 class LockError(Exception):
-    """All exceptions from the lock/unlock functions should be from
-    this exception class.  They will be translated as necessary. The
-    original exception is available as e.original_error
-    """
-    def __init__(self, e=None):
-        self.original_error = e
-        if e:
-            Exception.__init__(self, e)
-        else:
-            Exception.__init__(self)
+    """Lock error"""
+    # All exceptions from the lock/unlock functions should be from
+    # this exception class.  They will be translated as necessary. The
+    # original exception is available as e.original_error
 
 
 class CommitNotPossible(LockError):
@@ -123,8 +177,8 @@
     """A write attempt was made in a read only transaction."""
 
 
-class PointlessCommit(Exception):
-    """Commit failed because nothing was changed."""
+class PointlessCommit(BzrNewError):
+    """No changes to commit"""
 
 
 class NoSuchRevision(BzrError):

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-10-16 05:15:58 +0000
+++ bzrlib/inventory.py	2005-10-18 06:30:47 +0000
@@ -14,6 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# FIXME: This refactoring of the workingtree code doesn't seem to keep 
+# the WorkingTree's copy of the inventory in sync with the branch.  The
+# branch modifies its working inventory when it does a commit to make
+# missing files permanently removed.
 
 # TODO: Maybe also keep the full path of the entry, and the children?
 # But those depend on its position within a particular inventory, and
@@ -31,12 +35,11 @@
 import types
 
 import bzrlib
-from bzrlib.errors import BzrError, BzrCheckError
-
 from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
                             appendpath, sha_strings)
 from bzrlib.trace import mutter
-from bzrlib.errors import NotVersionedError
+from bzrlib.errors import (NotVersionedError, InvalidEntryName,
+                           BzrError, BzrCheckError)
 
 
 class InventoryEntry(object):
@@ -229,12 +232,11 @@
         '123'
         >>> e = InventoryFile('123', 'src/hello.c', ROOT_ID)
         Traceback (most recent call last):
-        BzrCheckError: InventoryEntry name 'src/hello.c' is invalid
+        InvalidEntryName: Invalid entry name: src/hello.c
         """
         assert isinstance(name, basestring), name
         if '/' in name or '\\' in name:
-            raise BzrCheckError('InventoryEntry name %r is invalid' % name)
-        
+            raise InvalidEntryName(name=name)
         self.executable = False
         self.revision = None
         self.text_sha1 = None
@@ -885,8 +887,7 @@
         parent_path = parts[:-1]
         parent_id = self.path2id(parent_path)
         if parent_id == None:
-            raise NotVersionedError(parent_path)
-
+            raise NotVersionedError(path=parent_path)
         if kind == 'directory':
             ie = InventoryDirectory(file_id, parts[-1], parent_id)
         elif kind == 'file':

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-10-17 05:18:20 +0000
+++ bzrlib/osutils.py	2005-10-18 02:31:11 +0000
@@ -119,7 +119,22 @@
         return F(f)
     else:
         return os.path.join(F(p), e)
-    
+
+if os.name == "posix":
+    # In Python 2.4.2 and older, os.path.abspath and os.path.realpath
+    # choke on a Unicode string containing a relative path if
+    # os.getcwd() returns a non-sys.getdefaultencoding()-encoded
+    # string.
+    _fs_enc = sys.getfilesystemencoding()
+    def abspath(path):
+        return os.path.abspath(path.encode(_fs_enc)).decode(_fs_enc)
+    def realpath(path):
+        return os.path.realpath(path.encode(_fs_enc)).decode(_fs_enc)
+else:
+    # We need to use the Unicode-aware os.path.abspath and
+    # os.path.realpath on Windows systems.
+    abspath = os.path.abspath
+    realpath = os.path.realpath
 
 def backup_file(fn):
     """Copy a file to a backup.
@@ -453,7 +468,7 @@
     os.path.commonprefix (python2.4) has a bad bug that it works just
     on string prefixes, assuming that '/u' is a prefix of '/u2'.  This
     avoids that problem."""
-    rp = os.path.abspath(path)
+    rp = abspath(path)
 
     s = []
     head = rp

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-10-18 12:07:55 +0000
+++ bzrlib/selftest/__init__.py	2005-10-18 13:11:57 +0000
@@ -181,7 +181,14 @@
         self._enable_file_logging()
 
     def _ndiff_strings(self, a, b):
-        """Return ndiff between two strings containing lines."""
+        """Return ndiff between two strings containing lines.
+        
+        A trailing newline is added if missing to make the strings
+        print properly."""
+        if b and b[-1] != '\n':
+            b += '\n'
+        if a and a[-1] != '\n':
+            a += '\n'
         difflines = difflib.ndiff(a.splitlines(True),
                                   b.splitlines(True),
                                   linejunk=lambda x: False,
@@ -553,10 +560,14 @@
                    'bzrlib.selftest.testannotate',
                    'bzrlib.selftest.testrevprops',
                    'bzrlib.selftest.testoptions',
+                   'bzrlib.selftest.testhttp',
+                   'bzrlib.selftest.testnonascii',
                    ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,
-              bzrlib.osutils, bzrlib.commands, bzrlib.merge3):
+              bzrlib.osutils, bzrlib.commands, bzrlib.merge3,
+              bzrlib.errors,
+              ):
         if m not in MODULES_TO_DOCTEST:
             MODULES_TO_DOCTEST.append(m)
 

=== modified file 'bzrlib/selftest/test_commit.py'
--- bzrlib/selftest/test_commit.py	2005-10-18 05:26:22 +0000
+++ bzrlib/selftest/test_commit.py	2005-10-18 13:11:57 +0000
@@ -19,6 +19,7 @@
 
 from bzrlib.selftest import TestCaseInTempDir
 from bzrlib.branch import Branch
+from bzrlib.workingtree import WorkingTree
 from bzrlib.commit import Commit
 from bzrlib.config import BranchConfig
 from bzrlib.errors import PointlessCommit, BzrError, SigningFailed
@@ -213,13 +214,15 @@
 
         
     def test_removed_commit(self):
-        """Test a commit with a removed file"""
+        """Commit with a removed file"""
         b = Branch.initialize('.')
+        wt = b.working_tree()
         file('hello', 'w').write('hello world')
         b.add(['hello'], ['hello-id'])
         b.commit(message='add hello')
 
-        b.working_tree().remove('hello')
+        wt = b.working_tree()  # FIXME: kludge for aliasing of working inventory
+        wt.remove('hello')
         b.commit('removed hello', rev_id='rev2')
 
         tree = b.revision_tree('rev2')
@@ -256,6 +259,24 @@
         # FIXME: This should raise a KeyError I think, rbc20051006
         self.assertRaises(BzrError, inv.__getitem__, 'file2id')
 
+    def test_strict_commit(self):
+        """Try and commit with unknown files and strict = True, should fail."""
+        from bzrlib.errors import StrictCommitFailed
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add('hello')
+        file('goodbye', 'w').write('goodbye cruel world!')
+        self.assertRaises(StrictCommitFailed, b.commit,
+            message='add hello but not goodbye', strict=True)
+
+    def test_nonstrict_commit(self):
+        """Try and commit with unknown files and strict = False, should work."""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add('hello')
+        file('goodbye', 'w').write('goodbye cruel world!')
+        b.commit(message='add hello but not goodbye', strict=False)
+
     def test_signed_commit(self):
         import bzrlib.gpg
         import bzrlib.commit as commit
@@ -297,5 +318,3 @@
             self.failIf(branch.revision_store.has_id('B'))
         finally:
             bzrlib.gpg.GPGStrategy = oldstrategy
-
-

=== modified file 'bzrlib/selftest/test_smart_add.py'
--- bzrlib/selftest/test_smart_add.py	2005-10-03 15:19:25 +0000
+++ bzrlib/selftest/test_smart_add.py	2005-10-18 06:30:47 +0000
@@ -3,7 +3,7 @@
 
 from bzrlib.selftest import TestCaseInTempDir, TestCase
 from bzrlib.branch import Branch
-from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.errors import NotBranchError
 from bzrlib.inventory import InventoryFile
 
 class TestSmartAdd(TestCaseInTempDir):

=== added file 'bzrlib/selftest/testhttp.py'
--- bzrlib/selftest/testhttp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testhttp.py	2005-10-18 02:08:30 +0000
@@ -0,0 +1,35 @@
+# (C) 2005 Canonical
+
+from bzrlib.selftest import TestCase
+from bzrlib.transport.http import HttpTransport
+
+class TestHttpUrls(TestCase):
+    def test_abs_url(self):
+        """Construction of absolute http URLs"""
+        t = HttpTransport('http://bazaar-ng.org/bzr/bzr.dev/')
+        eq = self.assertEqualDiff
+        eq(t.abspath('.'),
+           'http://bazaar-ng.org/bzr/bzr.dev')
+        eq(t.abspath('foo/bar'), 
+           'http://bazaar-ng.org/bzr/bzr.dev/foo/bar')
+        eq(t.abspath('.bzr'),
+           'http://bazaar-ng.org/bzr/bzr.dev/.bzr')
+        eq(t.abspath('.bzr/1//2/./3'),
+           'http://bazaar-ng.org/bzr/bzr.dev/.bzr/1/2/3')
+
+    def test_invalid_http_urls(self):
+        """Trap invalid construction of urls"""
+        t = HttpTransport('http://bazaar-ng.org/bzr/bzr.dev/')
+        self.assertRaises(ValueError,
+            t.abspath,
+            '.bzr/')
+        self.assertRaises(ValueError,
+            t.abspath,
+            '/.bzr')
+
+    def test_http_root_urls(self):
+        """Construction of URLs from server root"""
+        t = HttpTransport('http://bzr.ozlabs.org/')
+        eq = self.assertEqualDiff
+        eq(t.abspath('.bzr/tree-version'),
+           'http://bzr.ozlabs.org/.bzr/tree-version')

=== added file 'bzrlib/selftest/testnonascii.py'
--- bzrlib/selftest/testnonascii.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testnonascii.py	2005-10-18 02:33:49 +0000
@@ -0,0 +1,38 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test that various operations work in a non-ASCII environment."""
+
+import os
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+
+
+class NonAsciiTest(TestCaseInTempDir):
+
+    def test_add_in_nonascii_branch(self):
+        """Test adding in a non-ASCII branch."""
+        br_dir = u"\u1234"
+        try:
+            os.mkdir(br_dir)
+            os.chdir(br_dir)
+        except EncodingError:
+            self.log("filesystem can't accomodate nonascii names")
+            return
+        br = Branch.initialize(u".")
+        file("a", "w").write("hello")
+        br.add(["a"], ["a-id"])

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2005-10-10 05:12:40 +0000
+++ bzrlib/trace.py	2005-10-18 02:14:05 +0000
@@ -161,15 +161,16 @@
     The exception string representation is used as the error
     summary, unless msg is given.
     """
-    command = ' '.join(repr(arg) for arg in sys.argv)
-    prefix = "command: %s\npwd: %s\n" % (command, os.getcwd())
+    cmd_repr = ' '.join(repr(arg) for arg in sys.argv)
+    cmd_info = '\n  command: %s\n  pwd: %s' \
+        % (cmd_repr, os.getcwd())
     if msg == None:
         ei = sys.exc_info()
         msg = str(ei[1])
     if msg and (msg[-1] == '\n'):
         msg = msg[:-1]
     ## msg = "(%s) %s" % (str(type(ei[1])), msg)
-    _bzr_logger.exception(prefix + msg)
+    _bzr_logger.exception(msg + cmd_info)
 
 
 

=== modified file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	2005-10-16 14:19:24 +0000
+++ bzrlib/transport/http.py	2005-10-18 06:30:47 +0000
@@ -28,9 +28,6 @@
 from bzrlib.branch import Branch
 from bzrlib.trace import mutter
 
-# velocitynet.com.au transparently proxies connections and thereby
-# breaks keep-alive -- sucks!
-
 
 def get_url(url):
     import urllib2
@@ -78,24 +75,31 @@
         This can be supplied with a string or a list
         """
         if isinstance(relpath, basestring):
-            relpath = [relpath]
+            relpath_parts = relpath.split('/')
+        else:
+            # TODO: Don't call this with an array - no magic interfaces
+            relpath_parts = relpath[:]
+        if len(relpath_parts) > 1:
+            if relpath_parts[0] == '':
+                raise ValueError("path %r within branch %r seems to be absolute"
+                                 % (relpath, self._path))
+            if relpath_parts[-1] == '':
+                raise ValueError("path %r within branch %r seems to be a directory"
+                                 % (relpath, self._path))
         basepath = self._path.split('/')
         if len(basepath) > 0 and basepath[-1] == '':
             basepath = basepath[:-1]
-
-        for p in relpath:
+        for p in relpath_parts:
             if p == '..':
-                if len(basepath) < 0:
+                if len(basepath) == 0:
                     # In most filesystems, a request for the parent
                     # of root, just returns root.
                     continue
-                if len(basepath) > 0:
-                    basepath.pop()
-            elif p == '.':
+                basepath.pop()
+            elif p == '.' or p == '':
                 continue # No-op
             else:
                 basepath.append(p)
-
         # Possibly, we could use urlparse.urljoin() here, but
         # I'm concerned about when it chooses to strip the last
         # portion of the path, and when it doesn't.

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2005-10-17 23:13:00 +0000
+++ bzrlib/transport/local.py	2005-10-18 06:30:47 +0000
@@ -13,8 +13,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-"""Implementation of Transport for the local filesystem.
-"""
+
+"""Transport for the local filesystem.
+
+This is a fairly thin wrapper on regular file IO."""
 
 import os
 import errno
@@ -25,7 +27,7 @@
 from bzrlib.trace import mutter
 from bzrlib.transport import Transport, register_transport, \
     TransportError, NoSuchFile, FileExists
-
+from bzrlib.osutils import abspath
 
 class LocalTransportError(TransportError):
     pass
@@ -41,7 +43,7 @@
         # realpath is incompatible with symlinks. When we traverse
         # up we might be able to normpath stuff. RBC 20051003
         super(LocalTransport, self).__init__(
-            os.path.normpath(os.path.abspath(base)))
+            os.path.normpath(abspath(base)))
 
     def should_cache(self):
         return False

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-10-18 06:56:24 +0000
+++ bzrlib/workingtree.py	2005-10-18 13:11:57 +0000
@@ -14,11 +14,33 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-# TODO: Don't allow WorkingTrees to be constructed for remote branches.
+"""WorkingTree object and friends.
+
+A WorkingTree represents the editable working copy of a branch.
+Operations which represent the WorkingTree are also done here, 
+such as renaming or adding files.  The WorkingTree has an inventory 
+which is updated by these operations.  A commit produces a 
+new revision based on the workingtree and its inventory.
+
+At the moment every WorkingTree has its own branch.  Remote
+WorkingTrees aren't supported.
+
+To get a WorkingTree, call Branch.working_tree():
+"""
+
+
+# TODO: Don't allow WorkingTrees to be constructed for remote branches if 
+# they don't work.
 
 # FIXME: I don't know if writing out the cache from the destructor is really a
-# good idea, because destructors are considered poor taste in Python, and
-# it's not predictable when it will be written out.
+# good idea, because destructors are considered poor taste in Python, and it's
+# not predictable when it will be written out.
+
+# TODO: Give the workingtree sole responsibility for the working inventory;
+# remove the variable and references to it from the branch.  This may require
+# updating the commit code so as to update the inventory within the working
+# copy, and making sure there's only one WorkingTree for any directory on disk.
+# At the momenthey may alias the inventory and have old copies of it in memory.
 
 import os
 import stat
@@ -27,7 +49,7 @@
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock, quotefn
 import bzrlib.tree
 from bzrlib.osutils import appendpath, file_kind, isdir, splitpath, relpath
-from bzrlib.errors import BzrCheckError, DivergedBranches
+from bzrlib.errors import BzrCheckError, DivergedBranches, NotVersionedError
 from bzrlib.trace import mutter
 
 class TreeEntry(object):
@@ -105,9 +127,12 @@
         """
         from bzrlib.hashcache import HashCache
         from bzrlib.trace import note, mutter
-
+        assert isinstance(basedir, basestring), \
+            "base directory %r is not a string" % basedir
         if branch is None:
             branch = Branch.open(basedir)
+        assert isinstance(branch, Branch), \
+            "branch %r is not a Branch" % branch
         self._inventory = branch.inventory
         self.path2id = self._inventory.path2id
         self.branch = branch
@@ -454,7 +479,9 @@
         for f in files:
             fid = inv.path2id(f)
             if not fid:
-                raise BzrError("cannot remove unversioned file %s" % quotefn(f))
+                # TODO: Perhaps make this just a warning, and continue?
+                # This tends to happen when 
+                raise NotVersionedError(path=f)
             mutter("remove inventory entry %s {%s}" % (quotefn(f), fid))
             if verbose:
                 # having remove it, it must be either ignored or unknown

